/*================================================================
  File:     02_Basics.sql

  Summary:  Queries to explore basic information in plans

  Latest SQL Server Versions tested: SQL Server 2014 12.0.4100
------------------------------------------------------------
  Modified by Erin Stellato, SQLskills.com

  (c) 2015, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
=================================================================*/

USE [AdventureWorks2014];
GO

/*
	control flow vs. data flow
	operators, details
*/
SELECT TOP 25
	[LastName],
	[FirstName],
	[PersonType],
	[ModifiedDate]
FROM [Person].[Person]
WHERE [LastName] = 'Stewart';




/*
	** Execute two SELECTs together **
	Compare costs between statements, and within
	Total cost is in the SELECT
	Costs per operator
	Not every operator has CPU and I/O cost	
*/
SELECT TOP 25
	[LastName],
	[FirstName],
	[PersonType],
	[ModifiedDate]
FROM [Person].[Person]
WHERE [LastName] = 'Stewart';

SELECT 
  [BusinessEntityID], 
		  [LoginID], 
  [HireDate]
FROM [HumanResources].[Employee]
WHERE [BusinessEntityID] > 275
ORDER BY [HireDate];
GO


/*
	Clustered Index Seek -> 1 Row
	What's the Estimated I/O cost?
	--
*/
SELECT [LastName]
FROM [Person].[Person]
WHERE [BusinessEntityID] = 38;


/*
	Find the Page ID so we can check the cache
	NOTE: fn_PhysLocFormatter is undocumented (thus unsupported)
*/
SELECT  [LastName],
		sys.fn_PhysLocFormatter(%%physloc%%)
FROM [Person].[Person]
WHERE [BusinessEntityID] = 38;

SELECT *
FROM sys.dm_os_buffer_descriptors
WHERE page_id = 978 AND
		database_id = DB_ID();

/*
	Since it's cached, does cost change?
*/
SELECT [LastName]
FROM [Person].[Person]
WHERE [BusinessEntityID] = 38;


/*
	how does cost threshold for parallelism come into play?
*/
SELECT cpu_count AS [Logical CPU Count], 
cpu_count/hyperthread_ratio AS [Physical CPU Count]
FROM sys.dm_os_sys_info WITH (NOLOCK) OPTION (RECOMPILE);

SELECT *
FROM [sys].[configurations]
WHERE [name] IN ('cost threshold for parallelism', 'max degree of parallelism');

/*
	execute a parallel query
*/
USE [AdventureWorks2014];
GO

SELECT [d].[SalesOrderID], [d].[SalesOrderDetailID], [d].[OrderQty], [p].[ProductID], 
[d].[UnitPrice], [p].[Name], [p].[Color]
FROM [Sales].[SalesOrderDetail] [d]
JOIN [Production].[Product] [p] ON [d].[ProductID] = [p].[ProductID]
ORDER BY [Color];

/*
	change cost threshold for parallelism
*/
sp_configure 'cost threshold for parallelism', 25;
GO
RECONFIGURE;
GO

/*
	re-run
*/
SELECT [d].[SalesOrderID], [d].[SalesOrderDetailID], [d].[OrderQty], [p].[ProductID], 
[d].[UnitPrice], [p].[Name], [p].[Color]
FROM [Sales].[SalesOrderDetail] [d]
JOIN [Production].[Product] [p] ON [d].[ProductID] = [p].[ProductID]
ORDER BY [Color];


/*
	reset to default
*/
sp_configure 'cost threshold for parallelism', 5;
GO
RECONFIGURE;
GO

/*
	re-run and note DegreeOfParallelism
*/
SELECT [d].[SalesOrderID], [d].[SalesOrderDetailID], [d].[OrderQty], [p].[ProductID], 
[d].[UnitPrice], [p].[Name], [p].[Color]
FROM [Sales].[SalesOrderDetail] [d]
JOIN [Production].[Product] [p] ON [d].[ProductID] = [p].[ProductID]
ORDER BY [Color];


/*
	change maxdop
*/
sp_configure 'max degree of parallelism', 2;
GO
RECONFIGURE;
GO

/*
	re-run and check DegreeOfParallelism now
*/
SELECT [d].[SalesOrderID], [d].[SalesOrderDetailID], [d].[OrderQty], [p].[ProductID], 
[d].[UnitPrice], [p].[Name], [p].[Color]
FROM [Sales].[SalesOrderDetail] [d]
JOIN [Production].[Product] [p] ON [d].[ProductID] = [p].[ProductID]
ORDER BY [Color];

/*
	You can also control DOP with a hint...
	Note NonParallelPlanReason
*/
SELECT [d].[SalesOrderID], [d].[SalesOrderDetailID], [d].[OrderQty], [p].[ProductID], 
[d].[UnitPrice], [p].[Name], [p].[Color]
FROM [Sales].[SalesOrderDetail] [d]
JOIN [Production].[Product] [p] ON [d].[ProductID] = [p].[ProductID]
ORDER BY [Color]
OPTION (MAXDOP 1);